#include "AINetworkNodeSelection.h"
#include "AINetworkNode.h"
#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(AINetworkNodeSelection,AINetworkVertexSelection);

AINetworkNodeSelection::AINetworkNodeSelection(MapSelectionManager* manager, AINetworkNode* content):
        AINetworkVertexSelection(manager,content) {
}

AINetworkNodeSelection::~AINetworkNodeSelection() {
}

bool AINetworkNodeSelection::Hit(Point3D p) {
        return GetContent()->Hit(GetEditor(),TDRectangle(p,p));
}

MapSelectionHandle* AINetworkNodeSelection::HitHandle(Point3D p) {
        if (!HandlesActive())
                return (MapSelectionHandle*)NULL;
        // TODO: implement your own handles or remove this method
        return MapSelection::HitHandle(p);
}

wxRect AINetworkNodeSelection::GetDeviceBounds() const {
        wxRect r=GetContent()->GetDeviceBounds(GetEditor());
        if (HandlesActive()) {
                // TODO: incorporate handles bounds here
        }
        return r;
}

void AINetworkNodeSelection::Paint(wxDC& dc) {
        GetContent()->PaintNode(dc,GetEditor(),Selection_Marking_Colour);
}

void AINetworkNodeSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        AINetworkVertexSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkNodeSelection)))
                return;
        
        propgrid->Append(wxBoolProperty(wxT("Is on Runway?"), wxT("AINetworkNodeSelection_IsOnRunway")));
        
        wxPGChoices holdPointTypeChoices;
        holdPointTypeChoices.Add(wxT("None"),AINetworkNode::HoldPointType_None);
        holdPointTypeChoices.Add(wxT("Normal Holding Point"),AINetworkNode::HoldPointType_Normal);
        holdPointTypeChoices.Add(wxT("CAT II/III Holding Point"),AINetworkNode::HoldPointType_CAT_II_III);
        holdPointTypeChoices.Add(wxT("PushBack Holding Point"),AINetworkNode::HoldPointType_PushBack);
        
        propgrid->Append(wxEnumProperty(wxT("Holding Point Type"),
                                        wxT("AINetworkNodeSelection_HoldPointType"),
                                        holdPointTypeChoices));
}

void AINetworkNodeSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        AINetworkVertexSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkNodeSelection)))
                return;

        propgrid->SetPropertyValue(wxT("AINetworkNodeSelection_IsOnRunway"),GetContent()->IsOnRunway());
        propgrid->SetPropertyValue(wxT("AINetworkNodeSelection_HoldPointType"),GetContent()->GetHoldPointType());
}

void AINetworkNodeSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        AINetworkVertexSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkNodeSelection)))
                return;
        AINetworkNode* firstContent=wxDynamicCast(firstSelection,AINetworkNodeSelection)->GetContent();
        AINetworkNode* myContent=GetContent();

        if (firstContent->IsOnRunway()!=myContent->IsOnRunway())
                propgrid->SetPropertyUnspecified(wxT("AINetworkNodeSelection_IsOnRunway"));
        if (firstContent->GetHoldPointType()!=myContent->GetHoldPointType())
                propgrid->SetPropertyUnspecified(wxT("AINetworkNodeSelection_HoldPointType"));
}

void AINetworkNodeSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("AINetworkNodeSelection_IsOnRunway"))==0) {
                GetContent()->SetIsOnRunway(value.GetBool());
        } else if (name.Cmp(wxT("AINetworkNodeSelection_HoldPointType"))==0) {
                GetContent()->SetHoldPointType((AINetworkNode::HoldPointType)value.GetLong());
        } else {
                AINetworkVertexSelection::UpdateObjectFromProperty(property);
        }
}

