#ifndef Airport_h
#define Airport_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <wx/dynarray.h>

#include <Map/MapObjectContainer.h>
#include <Transactions/TransactionObject.h>

#include "FrequencyEntry.h"

class TransactionManager;

WX_DECLARE_OBJARRAY(FrequencyEntry,FrequencyEntryArray);

class Airport: public TransactionObject {
        
        DECLARE_CLASS(Airport)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(Airport);
        
public:
        typedef enum {
                AirportType_Land,
                AirportType_Sea,
                AirportType_Heliport
        } AirportType;
        
	Airport(TransactionManager* tmgr,
                const wxString& icaoId,
                const wxString& name,
                AirportType type,
                float elevation,
                bool hasControlTower,
                bool hasDefaultBuildings);
        virtual ~Airport();
        
        MapObjectContainer* GetRunwayLayer() const { return _runwayLayer; }
        MapObjectContainer* GetTaxiwayLayer() const { return _taxiwayLayer; }
        MapObjectContainer* GetObjectsLayer() const { return _objectsLayer; }
        MapObjectContainer* GetGroundNetworkLayer() const { return _groundNetworkLayer; }
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(const wxString&,ICAOID,_icaoId,
                                                TRANSACTIONOBJECT_STATEVAR(_icaoId).Cmp(v)==0)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(const wxString&,Name,_name,
                                                TRANSACTIONOBJECT_STATEVAR(_name).Cmp(v)==0)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(AirportType,Type,_type)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(float,Elevation,_elevation)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(bool,ControlTower,_hasControlTower)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(bool,DefaultBuildings,_hasDefaultBuildings)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(const FrequencyEntryArray&,Frequencies,_frequencies)
        
        bool HasControlTower() const { return GetControlTower(); }
        bool HasDefaultBuildings() const { return GetDefaultBuildings(); }
protected:
        typedef struct {
                wxString _icaoId;
                wxString _name;
                AirportType _type;
                float _elevation;
                bool _hasControlTower;
                bool _hasDefaultBuildings;
                FrequencyEntryArray _frequencies;
        } State;
        
        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(Airport,TransactionObject,State)
protected:
        MapObjectContainer::SmartRef _runwayLayer;
        MapObjectContainer::SmartRef _taxiwayLayer;
        MapObjectContainer::SmartRef _objectsLayer;
        MapObjectContainer::SmartRef _groundNetworkLayer;
};

#endif

