#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "TaxiwaySelection.h"
#include "TaxiwayMapObject.h"
#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(TaxiwaySelection,PavementSelection);

TaxiwaySelection::TaxiwaySelection(MapSelectionManager* manager, TaxiwayMapObject* content):
        PavementSelection(manager,content) {
}

TaxiwaySelection::~TaxiwaySelection() {
}

void TaxiwaySelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PavementSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(TaxiwaySelection)))
                return;
        propgrid->Append(wxPropertyCategory(wxT("Lighting")));
        
        propgrid->Append(wxBoolProperty(wxT("Edge Lights"), wxT("TaxiwaySelection_EdgeLights")));
        propgrid->Append(wxBoolProperty(wxT("Center Lights"), wxT("TaxiwaySelection_CenterLights")));

        wxPGChoices surfaceChoices;
        surfaceChoices.Add(wxT("Asphalt"),PavementMapObject::Surface_Asphalt);
        surfaceChoices.Add(wxT("Concrete"),PavementMapObject::Surface_Concrete);
        surfaceChoices.Add(wxT("Turf"),PavementMapObject::Surface_Turf);
        surfaceChoices.Add(wxT("Dirt"),PavementMapObject::Surface_Dirt);
        surfaceChoices.Add(wxT("Gravel"),PavementMapObject::Surface_Gravel);
        surfaceChoices.Add(wxT("Dry Lakebed"),PavementMapObject::Surface_Lakebed);
        surfaceChoices.Add(wxT("Water"),PavementMapObject::Surface_Water);
        surfaceChoices.Add(wxT("Asphalt with yellow hold line"),PavementMapObject::Surface_Twy_Asphalt_yellow_hold);
        surfaceChoices.Add(wxT("Concrete with yellow hold line"),PavementMapObject::Surface_Twy_Concrete_yellow_hold);
        
        propgrid->SetPropertyChoices(wxT("PavementSelection_Surface"),surfaceChoices);
}

void TaxiwaySelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PavementSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(TaxiwaySelection)))
                return;

        propgrid->SetPropertyValue(wxT("TaxiwaySelection_EdgeLights"),GetContent()->HasEdgeLights());
        propgrid->SetPropertyValue(wxT("TaxiwaySelection_CenterLights"),GetContent()->HasCenterLights());
}

void TaxiwaySelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        PavementSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(TaxiwaySelection)))
                return;
        TaxiwayMapObject* firstContent=wxDynamicCast(firstSelection,TaxiwaySelection)->GetContent();
        TaxiwayMapObject* myContent=GetContent();

        if (firstContent->HasEdgeLights()!=myContent->HasEdgeLights())
                propgrid->SetPropertyUnspecified(wxT("TaxiwaySelection_EdgeLights"));
        if (firstContent->HasCenterLights()!=myContent->HasCenterLights())
                propgrid->SetPropertyUnspecified(wxT("TaxiwaySelection_CenterLights"));
}

void TaxiwaySelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("TaxiwaySelection_EdgeLights"))==0) {
                GetContent()->SetEdgeLights(value.GetBool());
        } else if (name.Cmp(wxT("TaxiwaySelection_CenterLights"))==0) {
                GetContent()->SetCenterLights(value.GetBool());
        } else {
                PavementSelection::UpdateObjectFromProperty(property);
        }
}


