#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapCalibratedBackgroundImage.h"
#include "MapCalibratedBackgroundImageSelection.h"
#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>

#include <cmath>

IMPLEMENT_CLASS(MapCalibratedBackgroundImage,MapSimpleBackgroundImage)

MapCalibratedBackgroundImage::MapCalibratedBackgroundImage(TransactionManager* transactionManager,
                                                           Point3D northWestPos,
                                                           double scale,
                                                           double angle):
        MapSimpleBackgroundImage(transactionManager,northWestPos,scale)
{
        TRANSACTIONOBJECT_STATEVAR(_angle)=angle;
}

MapCalibratedBackgroundImage::~MapCalibratedBackgroundImage() {
}

MapSelection* MapCalibratedBackgroundImage::CreateSelection(MapSelectionManager* manager) {
        return new MapCalibratedBackgroundImageSelection(manager,this);
}

void MapCalibratedBackgroundImage::UpdateCacheImage() {
        if (!_image.Ok())
                return;
        if (::fabs(GetAngle())<0.05) {
                _cacheImage=_image;
        } else {
                _cacheImage=_image.Rotate(-GetAngle(),wxPoint(0,0),true,&_imageOffset);
        }
}

const wxImage& MapCalibratedBackgroundImage::LoadImage() {
        _cacheImage.Destroy();
        return MapSimpleBackgroundImage::LoadImage();
}

void MapCalibratedBackgroundImage::GetLocalCorners(MapEditor* editor, Point3D corners[4]) {
        MapProjection* proj=editor->GetProjection();
        Point3D localNWPos=proj->ConvertToLocal(GetNorthWestPos());
        wxCoord width=0,height=0;
        
        if (_image.Ok()) {
                width=_image.GetWidth();
                height=_image.GetHeight();
        }
        double scale=GetScale();
        double angle=GetAngle();
        Point3D vx,vy;
        vx=Point3D( ::cos(angle)*scale,-::sin(angle)*scale,0.0);
        vy=Point3D(-::sin(angle)*scale,-::cos(angle)*scale,0.0);
        corners[0]=localNWPos;
        corners[1]=localNWPos+width*vx;
        corners[2]=localNWPos+width*vx+height*vy;
        corners[3]=localNWPos+height*vy;
}

