#ifndef MapCalibratedBackgroundImage_h
#define MapCalibratedBackgroundImage_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "MapSimpleBackgroundImage.h"

class MapCalibratedBackgroundImage: public MapSimpleBackgroundImage {
        
        DECLARE_CLASS(MapCalibratedBackgroundImage)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(MapCalibratedBackgroundImage)
        
public:
        /* Create a new object */
	MapCalibratedBackgroundImage(TransactionManager* transactionManager,
                                     Point3D northEastPos,
                                     double scale,
                                     double angle);
        
        /* May only be called by MapLayer */
        virtual ~MapCalibratedBackgroundImage();
        
        virtual MapSelection* CreateSelection(MapSelectionManager* manager);
        
        void SetCalibration(Point3D northWestPos,double scale,double angle) {
                SetNorthWestPos(northWestPos);
                SetScale(scale);
                SetAngle(angle);
        }
        
        TRANSACTIONOBJECT_DEFINE_STATE_GETTER(double,Angle,_angle);
        void SetAngle(double v) {
                if (TRANSACTIONOBJECT_STATEVAR(_angle)==v)
                        return;
                BeforeWrite();
                
                TRANSACTIONOBJECT_STATEVAR(_angle)=v;
                _cacheImage.Destroy();
                
                AfterWrite();
        }
        
        virtual const wxImage& LoadImage();
        virtual void GetLocalCorners(MapEditor* editor, Point3D corners[4]);
protected:
        typedef struct {
                double _angle;
        } State;

        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(MapCalibratedBackgroundImage,MapSimpleBackgroundImage,State)
        
        virtual const wxImage& GetImage() {
                if (!_cacheImage.Ok()) {
                        UpdateCacheImage();
                }
                return _cacheImage;
        }
        
        virtual const wxPoint GetImageOffset() { return _imageOffset; }

        void UpdateCacheImage();
        
        wxImage _cacheImage;
        wxPoint _imageOffset;
};

#endif

