#ifndef MapUSGSImageLayer_h
#define MapUSGSImageLayer_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <wx/filename.h>

#include "Map/MapObjectContainer.h"

#include <Projection/UTMProjection.h>

class MapUSGSImage;

WX_DECLARE_STRING_HASH_MAP( MapUSGSImage*, MapUSGSImageHashMap );

class MapUSGSImageLayer: public MapObjectContainer {
        
        DECLARE_CLASS(MapUSGSImageLayer)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(MapUSGSImageLayer)
public:
	MapUSGSImageLayer(TransactionManager* tmgr, wxString name);
        virtual ~MapUSGSImageLayer();

        typedef enum {
                ThemeType_DOQ=1,
                ThemeType_DRG=2,
                ThemeType_Urban=4
        } ThemeType;
        
        void FetchImagesForRectangle(const TDRectangle geoBounds,
                                     wxFileName imgPath);
        
        void CancelPendingRequests();
        
        static Point3D GetNorthWestPosForTile(int zone, int scale, int x, int y);
        
        static float GetResolutionForScale(int scale) {
                return pow(2.0,scale-10);
        }
        
        static float GetTilesizeForScale(int scale) {
                return 200*pow(2.0,scale-10);
        }
        
        void SetRequestScale(int scale) {
                _scale=scale;
        }
        
        int GetRequestScale() const {
                return _scale;
        }
        
        void SetRequestThemeType(ThemeType type) {
                _type=type;
        }
        
        ThemeType GetRequestThemeType() const {
                return _type;
        }
        
        static bool IsThemeScaleAvailable(ThemeType type, int scale);
        
        bool HasScale(int scale) {
                return IsThemeScaleAvailable(_type,scale);
        }
        
        bool HasThemeType(ThemeType type) {
                return IsThemeScaleAvailable(type,_scale);
        }
        
        virtual void DetachObject(MapObject* obj);

        static UTMProjection _utmProjection;
protected:
        typedef struct {
                MapUSGSImageHashMap _imagesByPath;
        } State;
        
        int _scale;
        ThemeType _type;
        
        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(MapUSGSImageLayer,MapObjectContainer,State)
};

#endif

