#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "ProjectWizardNewProjectPage.h"

#include "MapFrame.h"
#include "MapViewManager.h"

#include <IO/XPlaneLoader.h>
#include <Math/TDRectangle.h>
#include <Transactions/TransactionManager.h>

#include <wx/config.h>
#include <wx/dirctrl.h>
#include <wx/filename.h>

#include <fstream>

IMPLEMENT_CLASS(ProjectWizardNewProjectPage,wxWizardPageSimple);

enum {
        ProjectWizardNewProjectPage_ID_XPlaneImportPath=wxID_HIGHEST,
        ProjectWizardNewProjectPage_ID_XPlaneImportPathBrowse
};

BEGIN_EVENT_TABLE(ProjectWizardNewProjectPage,wxWizardPageSimple)
        EVT_BUTTON(ProjectWizardNewProjectPage_ID_XPlaneImportPathBrowse, ProjectWizardNewProjectPage::OnXPlanePathBrowse)
        EVT_WIZARD_PAGE_CHANGING(wxID_ANY,ProjectWizardNewProjectPage::OnPageChanging)
END_EVENT_TABLE()

ProjectWizardNewProjectPage::ProjectWizardNewProjectPage(wxWizard* parent,
                     wxWizardPage* prev,
                     wxWizardPage* next,
                     const wxBitmap& bitmap,
                     const wxChar *resource):
             wxWizardPageSimple(parent,prev,next,bitmap,resource)
{
        wxBoxSizer* topSizer=new wxBoxSizer(wxVERTICAL);
        
        wxStaticText* text;
        
        text=new wxStaticText(this,wxID_ANY,wxT("Enter the ID of the airport to import"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        _icaoID=new wxTextCtrl(this,wxID_ANY,wxT("KSFO"));
        topSizer->Add(_icaoID);

        wxBoxSizer* rowsizer=new wxBoxSizer(wxHORIZONTAL);
        _xplaneImportPath=new wxTextCtrl(this,ProjectWizardNewProjectPage_ID_XPlaneImportPath);
        _xplaneImportPathBrowse=new wxButton(this,ProjectWizardNewProjectPage_ID_XPlaneImportPathBrowse,wxT("..."));
        wxSize ctrlsize=_xplaneImportPath->GetSize();
        ctrlsize.SetWidth(400);
        _xplaneImportPath->SetMinSize(ctrlsize);
        rowsizer->Add(_xplaneImportPath,wxSizerFlags(1).Expand());
        rowsizer->Add(_xplaneImportPathBrowse,wxSizerFlags(0).Centre());
        topSizer->Add(rowsizer);
        
        SetSizer(topSizer);
        
        topSizer->SetSizeHints(this);
        
        Fit();
}

ProjectWizardNewProjectPage::~ProjectWizardNewProjectPage() {
}

bool ProjectWizardNewProjectPage::TransferDataToWindow() {
        if (!wxWizardPageSimple::TransferDataToWindow())
                return false;
        wxConfigBase* config=wxConfigBase::Get();
        
        wxString xplaneImportPathStr;
        if (config->Read(wxT("xplane-data-path"),&xplaneImportPathStr)) {
                _xplaneImportPath->SetValue(xplaneImportPathStr);
        }
        return true;
}

void ProjectWizardNewProjectPage::OnPageChanging(wxWizardEvent& evt) {
        if (!evt.GetDirection())
                return;
        wxConfigBase* config=wxConfigBase::Get();
        
        wxString xplaneImportPathStr=_xplaneImportPath->GetValue();
        if (xplaneImportPathStr.IsEmpty() || !::wxFileExists(xplaneImportPathStr)) {
                ::wxMessageBox(wxT("The X-Plane import file you specified does not exist.")
                               wxT("Please specify an existing file."),
                               wxT("X-Plane import database does not exist"),
                               wxOK|wxICON_HAND);
                evt.Veto();
                return;
        }
        
        config->Write(wxT("xplane-data-path"),xplaneImportPathStr);

        wxString icaoID=_icaoID->GetValue().MakeUpper();
        TransactionManager* tmgr=new TransactionManager();
        
        TDRectangle bounds;
        
        std::ifstream infile(xplaneImportPathStr.mb_str(*wxConvFileName));
        XPlaneLoader loader(tmgr,infile);
        
        Airport::SmartRef airport;
        airport=loader.loadAirport(std::string(icaoID.mb_str(wxConvUTF8)));
        if (!airport) {
                ::wxMessageBox(wxT("The airport with the given ID '")+icaoID+wxT("' ")
                               wxT("could not be loaded. Perhaps the airport does ")
                               wxT("not exist or the database file is corrupted."),
                               wxT("Unable to load airport '")+icaoID+wxT("'"),
                               wxOK|wxICON_ERROR);
                evt.Veto();
                return;
        }
        
        MapFrame* newFrame=new MapFrame(airport);
        MapEditor* newEditor=newFrame->GetEditor();
        
        bounds=newEditor->GetGeographicBounds();
        MapProjection* proj=newEditor->GetProjection();
        Point3D center=bounds.GetCenter();
        proj->Init(center);
        
        newEditor->GetTransactionManager()->Clear();
        
        bounds=newEditor->GetLocalBounds();
        newEditor->ZoomTo(bounds);
        
        newFrame->Show();
        MapViewManager::GetInstance()->ActivateView(newFrame);
}

void ProjectWizardNewProjectPage::OnXPlanePathBrowse(wxCommandEvent&) {
        wxString defFile=_xplaneImportPath->GetValue();
        wxFileName defFilename=wxFileName(defFile);
        
        const wxString& file = ::wxFileSelector(wxT("Choose X-Plane database file"),
                defFilename.GetPath(),
                defFilename.GetFullName(),
                wxT("dat"),
                wxT("X-Plane database file (*.dat)|*.dat|All files|*.*"),wxOPEN|wxFILE_MUST_EXIST);
        
        if (file.IsEmpty())
                return;
        
        _xplaneImportPath->SetValue(file);
}

