#ifndef MapDraggable_h
#define MapDraggable_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Math/point3d.h>

/**
 * \brief A MapDraggable is an object that can be dragged on the map.
 */
class MapDraggable {
public:
        virtual ~MapDraggable() {}
        
        /**
         * \brief Start dragging this draggable.
         *
         * \param currentPosition       The current position of the mouse.
         *
         * The position is given in the local coordinate system of the editor
         * the selection is associated with.
         */
        virtual void StartDrag(Point3D currentPosition)=0;
        
        /**
         * \brief Drag this draggable.
         *
         * \param currentPosition       The current position of the mouse.
         *
         * The positions are given in the local coordinate system of the editor
         * the selection is associated with.
         */
        virtual void Drag(Point3D currentPosition)=0;
        
        /**
         * \brief End dragging this draggable.
         *
         * \return A textual description of the modification applied by
         *         dragging the draggable.
         */
       virtual wxString EndDrag()=0;
};

#endif

