#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapGrid.h"
#include "MapEditor.h"
#include "MapCoordinateSystem.h"
#include <Map/MapProjection.h>

IMPLEMENT_CLASS(MapGrid,MapEditorDecoration);

MapGrid::MapGrid(MapEditor* editor):
        MapEditorDecoration(editor) {
}

MapGrid::~MapGrid() {
}

void MapGrid::Paint(wxDC& dc) {
        if (!IsEnabled())
                return;
        wxPen OrangePen=wxPen(wxColour(255,127,0),1,wxSOLID);
        wxPen DarkOrangePen=wxPen(wxColour(220,100,10),2,wxSOLID);
        
        dc.SetPen(OrangePen);
        DrawGrid(dc,5,false);
        dc.SetPen(DarkOrangePen);
        DrawGrid(dc,30,true);
        dc.SetPen(wxNullPen);
}

void MapGrid::DrawGrid(wxDC& dc, int res, bool degText) {
        wxSize clientSize=GetEditor()->GetDrawingWindow()->GetClientSize();
        MapCoordinateSystem* cs=&GetEditor()->GetCoordinateSystem();
        MapProjection* proj=GetEditor()->GetProjection();
        
        wxCoord top,bottom,left,right;

        left=dc.DeviceToLogicalX(0);
        right=dc.DeviceToLogicalX(clientSize.GetWidth());
        top=dc.DeviceToLogicalY(0);
        bottom=dc.DeviceToLogicalY(clientSize.GetHeight());
        
        Point3D topLeft,bottomRight;
        
        topLeft=cs->DeviceToLogical(wxPoint(left,top));
        bottomRight=cs->DeviceToLogical(wxPoint(right,bottom));
        
        Point3D NE,NW,SE,SW;
        
        NW=proj->ConvertFromLocal(topLeft);
        SE=proj->ConvertFromLocal(bottomRight);
        NE=proj->ConvertFromLocal(Point3D(bottomRight.x(),topLeft.y(),0));
        SW=proj->ConvertFromLocal(Point3D(topLeft.x(),bottomRight.y(),0));
        
        double minlat,maxlat,minlon,maxlon;
        minlat=std::min(SE.y(),SW.y());
        maxlat=std::max(NE.y(),NW.y());
        minlon=std::min(NW.x(),SW.x());
        maxlon=std::max(NE.x(),SE.x());
        
        int minSecLat,maxSecLat,minSecLon,maxSecLon;
        minSecLat=int(floor(minlat*3600.0/(double)res))*res;
        minSecLon=int(floor(minlon*3600.0/(double)res))*res;
        maxSecLat=int(ceil(maxlat*3600.0/(double)res))*res;
        maxSecLon=int(ceil(maxlon*3600.0/(double)res))*res;
        
        int cols;
        
        cols=(maxSecLon-minSecLon)/res+1;
        
        if (cols>300)
                return;
        
        wxPoint lastRow[300],gridpt;
        
        int seclat,seclon,i,j;
        
        _leftTextWidth=0;
        _topTextHeight=0;
        
        wxCoord w,h;
        
        for (seclat=minSecLat,i=0;seclat<=maxSecLat;seclat+=res,i++) {
                for (seclon=minSecLon,j=0;seclon<=maxSecLon;seclon+=res,j++) {
                        Point3D p=Point3D(seclon/3600.0,seclat/3600.0,0);
                        gridpt=cs->LogicalToDevice(proj->ConvertToLocal(p));
                        if (i>0) {
                                dc.DrawLine(lastRow[j].x,lastRow[j].y,
                                            gridpt.x,gridpt.y);
                        }
                        if (j>0) {
                                dc.DrawLine(lastRow[j-1].x,lastRow[j-1].y,
                                            gridpt.x,gridpt.y);
                        }
                        if (i==0 && degText) {
                                wxString txt=SecToText(seclon);
                                dc.GetTextExtent(txt,&w,&h);
                                _topTextHeight=std::max(_topTextHeight,h);
                                dc.DrawText(txt,gridpt.x+2,top+2);
                        }
                        if (j==0 && degText) {
                                wxString txt=SecToText(seclat);
                                dc.GetTextExtent(txt,&w,&h);
                                _leftTextWidth=std::max(_leftTextWidth,w);
                                dc.DrawText(txt,left+2,gridpt.y+2);
                        }
                        lastRow[j]=gridpt;
                }
        }
}

void MapGrid::Scroll(wxSize devDelta) {
        wxSize clientSize=GetEditor()->GetDrawingWindow()->GetClientSize();
        
        GetEditor()->DamageRect(wxRect(devDelta.GetWidth(),devDelta.GetHeight(),
                                       clientSize.GetWidth()+devDelta.GetWidth(),
                                       _topTextHeight+2+devDelta.GetHeight()));
        GetEditor()->DamageRect(wxRect(devDelta.GetWidth(),devDelta.GetHeight(),
                                       _leftTextWidth+2+devDelta.GetWidth(),
                                       clientSize.GetHeight()+devDelta.GetHeight()));
}

wxString MapGrid::SecToText(int secs) {
        int degs,mins;
        secs=abs(secs);
        degs=secs/3600;
        mins=secs/60-degs*60;
        secs=secs-mins*60-degs*3600;
        char buffer[100];
        int result=sprintf(buffer,"%i:%02i:%02i",degs,mins,secs);
        assert(result>0 && result<100);
        return wxString(buffer, wxConvLibc);
}
