#include "CoordinateDisplayMode.h"

#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>

IMPLEMENT_CLASS(CoordinateDisplayMode,Mode)

BEGIN_EVENT_TABLE(CoordinateDisplayMode,Mode)
        EVT_MOTION(CoordinateDisplayMode::OnMouseMotion)
END_EVENT_TABLE()

CoordinateDisplayMode::CoordinateDisplayMode(wxStatusBar* statusbar, int field):
        _statusbar(statusbar),
        _field(field)
{
}

CoordinateDisplayMode::~CoordinateDisplayMode() {
}

bool CoordinateDisplayMode::CanExit() {
        return false;
}

void CoordinateDisplayMode::OnMouseMotion(wxMouseEvent& evt) {
        MapEditor* editor=GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        MapProjection* proj=editor->GetProjection();
        
        wxPoint devPos=evt.GetPosition();
        Point3D logicalPos=cs.DeviceToLogical(devPos);
        Point3D geoPos=proj->ConvertFromLocal(logicalPos);
        
        int latDeg,latMin,lonDeg,lonMin;
        float latSec,lonSec;
        
        latDeg=(int)geoPos.lat();
        latMin=(int)((geoPos.lat()-latDeg)*60.0);
        latSec=(geoPos.lat()-latDeg-latMin/60.0)*3600.0;
        lonDeg=(int)geoPos.lon();
        lonMin=(int)((geoPos.lon()-lonDeg)*60.0);
        lonSec=(geoPos.lon()-lonDeg-lonMin/60.0)*3600.0;
        
        wxString position;
        position.Printf(wxT("%c%02dd%02dm%06.3fs %c%03dd%02dm%06.3fs x=%011.4f y=%011.4f"),
                        (latDeg<0?'S':'N'),abs(latDeg),abs(latMin),fabs(latSec),
                        (lonDeg<0?'W':'E'),abs(lonDeg),abs(lonMin),fabs(lonSec),
                        logicalPos.x(),logicalPos.y());
        _statusbar->SetStatusText(position,_field);
}
