#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "PopupMode.h"
#include <Map/MapEditor.h>
#include <Map/MapObject.h>
#include <Map/MapSelectionManager.h>
#include <Map/MapSelectionHandle.h>

IMPLEMENT_CLASS(PopupMode,Mode)

BEGIN_EVENT_TABLE(PopupMode,Mode)
        EVT_CONTEXT_MENU(PopupMode::OnContextMenu)
END_EVENT_TABLE()

PopupMode::PopupMode() {
}

/* Return TRUE iff this mode can be exited. */
bool PopupMode::CanExit() {
        return false;
}

void PopupMode::OnContextMenu(wxContextMenuEvent& evt) {
        wxWindow* client=GetEditor()->GetDrawingWindow();
        MapSelectionManager &selMgr=GetEditor()->GetSelectionManager();
        wxPoint clientPos=client->ScreenToClient(evt.GetPosition());
        
        wxMenu popupMenu(wxT("Actions"));
        
	popupMenu.SetEventHandler(&popupMenu);
        popupMenu.SetClientData(GetEditor());
        
        size_t separatorPos;
        // Allow the editor to specify its own actions
        GetEditor()->ProvideActionMenu(popupMenu);
        
        // Allow the objects to provide their actions
        if (selMgr.GetSelections().GetCount()!=0) {
                wxClassInfo* commonClass=selMgr.FindCommonSelectionSuperClass();
                MapSelection* handlerSelection=selMgr.FindFirstSelectionOfType(commonClass);
                separatorPos=popupMenu.GetMenuItemCount();
                handlerSelection->ProvideActionMenuForClass(popupMenu,commonClass);
                if (separatorPos>0 && separatorPos!=popupMenu.GetMenuItemCount())
                        popupMenu.InsertSeparator(separatorPos);
        }
        
        // Allow any possibly hit map selection handle to provide its actions
        Point3D localPos;
        localPos=GetEditor()->GetCoordinateSystem().DeviceToLogical(clientPos);
        MapSelectionHandle* handle=selMgr.HitHandle(localPos);
        if (handle) {
                separatorPos=popupMenu.GetMenuItemCount();
                handle->ProvideActionMenu(popupMenu);
                if (separatorPos>0 && separatorPos!=popupMenu.GetMenuItemCount())
                        popupMenu.InsertSeparator(separatorPos);
        }
        
        /* Might be that neither the editor nor the objects have added
         * any actions. In that case we don't display the menu */
        if (popupMenu.GetMenuItemCount()!=0)
                client->PopupMenu(&popupMenu,clientPos);
}
