#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "UTMProjection.h"
#include "datum.h"
#include "utm.h"

#include <iostream>

using namespace std;

IMPLEMENT_DYNAMIC_CLASS(UTMProjection,MapProjection)

MapDatumProjectionDescriptor UTMProjection::UTM_NAD83(wxT("UTM_NAD83"),
                                                      wxT("UTM with NAD83 ellipsoid"),
                                                      CLASSINFO(UTMProjection),
                                                      DatumGRS80);

void UTMProjection::RegisterProjections() {
        MapProjectionDescriptor::RegisterDescriptor(&UTM_NAD83);
}

UTMProjection::UTMProjection():
        _datum(DatumGRS80)              // Default is GRS80 as that is what the USGS stuff uses
{
        SetDescriptor(&UTM_NAD83);
}

UTMProjection::~UTMProjection() {
}

bool UTMProjection::Init(Point3D centre) {
        MapDatumProjectionDescriptor* desc;
        desc=wxDynamicCast(GetDescriptor(),MapDatumProjectionDescriptor);
        wxASSERT(desc);
        _datum=datum(desc->GetDatum());
	double init_x;
	double init_y;
	long result = redfearn(&_datum, &init_x, &init_y, &_zone, centre.lat(), centre.lon(), 1);
	_origin = centre;
	return(0 == result ? true : false);
}

bool UTMProjection::SetZone(int zone) {
	/*
	double init_x;
	double init_y;
	long result = redfearn(&_datum, &init_x, &init_y, &_zone, _origin.lat(), _origin.lon(), 1);
	if(0 == result) {
		_zone = zone;
		return(true);
	}
	*/
	_zone = zone;
	return(false);
}

int UTMProjection::GetZone(double lat, double lon) {
	double x;
	double y;
	long zone;
	long result = redfearn(&_datum, &x, &y, &zone, lat, lon, 1);
	if(result != 0) {
		return(-99);
	}
	return(zone);
}

Point3D UTMProjection::ConvertToLocal(Point3D pt) {
	double x;
	double y;
	long zone = (_zone < 0 ? _zone * -1 : _zone);
	long result = redfearn(&_datum, &x, &y, &zone, pt.lat(), pt.lon(), 1, false);
	if(result != 0) {
		return(Point3D(-9999.9, -9999.9, 0.0));
	}
	return(Point3D(x, y, 0.0));
}

Point3D UTMProjection::ConvertFromLocal(Point3D pt) {
	double lat;
	double lon;
	long result = redfearn_inverse(&_datum, pt.x(), pt.y(), _zone, &lat, &lon);
	if(result != 0) {
		return(Point3D(-99.9, -99.9, 0.0));
	}
	return(Point3D(lon, lat, 0.0));
}

Point3D UTMProjection::ConvertToLocal(Point3D pt, bool sticky_zone) {
	double x;
	double y;
	long zone = (_zone < 0 ? _zone * -1 : _zone);
	long result = redfearn(&_datum, &x, &y, &zone, pt.lat(), pt.lon(), 1, sticky_zone);
	if(result != 0) {
		return(Point3D(-9999.9, -9999.9, 0.0));
	}
	return(Point3D(x, y, 0.0));
}

Point3D UTMProjection::ConvertFromLocal(Point3D pt, int zone) {
	double lat;
	double lon;
	long result = redfearn_inverse(&_datum, pt.x(), pt.y(), zone, &lat, &lon);
	if(result != 0) {
		return(Point3D(-99.9, -99.9, 0.0));
	}
	return(Point3D(lon, lat, 0.0));
}

Point3D UTMProjection::ConvertToLocal(Point3D pt, int zone) {
	double x;
	double y;
	long z = (long)zone;
	long result = redfearn(&_datum, &x, &y, &z, pt.lat(), pt.lon(), 1, false);
	if(result != 0) {
		return(Point3D(-9999.9, -9999.9, 0.0));
	}
	return(Point3D(x, y, 0.0));
}

Point3D UTMProjection::GetZoneSpecificLocal(Point3D pt, int zone1, int zone2) {
	Point3D pos = ConvertFromLocal(pt, zone1);
	return(ConvertToLocal(pos, zone2));
}
