// UTMProjection.hxx - A class to encapsulate the UTM projection.
//
// Written by David Luff, started 2003.
//
// Copyright (C) 2002  David C Luff - david.luff@nottingham.ac.uk
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef _UTM_PROJECTION_H
#define _UTM_PROJECTION_H

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Map/MapProjection.h>

#include <Map/MapDatumProjectionDescriptor.h>
#include "datum.h"
#include "utm.h"

class UTMProjection : public MapProjection {
        DECLARE_DYNAMIC_CLASS(UTMProjection)
public:
        UTMProjection();
        ~UTMProjection();
        
        // Returns true if sucessfull
        bool Init(Point3D centre);
        
        // Explicitly set the UTM zone.  Caveat Emptor!
        bool SetZone(int zone);
        
        // Convert a lat/lon co-ordinate to the local projection
        // Original zone from init is used.
        Point3D ConvertToLocal(Point3D pt);
        
        // Convert a local projection co-ordinate to lat/lon
        Point3D ConvertFromLocal(Point3D pt);
        
        // Convert a lat/lon co-ordinate to the local projection
        // sticky_zone: true => original zone from init is used, 
        //             false => zone gets calculated for supplied lat/lon
        // Note that false will NOT reset the original zone from init.
        Point3D ConvertToLocal(Point3D pt, bool sticky_zone);
        
        // Return the initial zone (will be the airport origin zone for TaxiDraw).
        inline int GetZone() { return((int)_zone); }
        
        // Return the UTM zone for an arbitrary location
        // Returns -99 if it fails.
        int GetZone(double lat, double lon);
        
        // Given UTM co-ordinates pt in zone1, return the UTM co-ordinates of
        // the same point relative to zone2.
        Point3D GetZoneSpecificLocal(Point3D pt, int zone1, int zone2);
        
        // Convert a lat/lon co-ordinate to the local projection for a given zone
        Point3D ConvertToLocal(Point3D pt, int zone);
        
        // Convert a local projection co-ordinate to lat/lon for a given zone
        Point3D ConvertFromLocal(Point3D pt, int zone);
        
        static void RegisterProjections();
        
        static MapDatumProjectionDescriptor UTM_NAD83;
private:
        Point3D _origin;	// lat/lon of local area origin
        //double correction_factor;	// Reduction in surface distance per degree of longitude due to latitude.  Saves having to do a cos() every call.
	
	datum _datum;
	long _zone;		// zone is determined from redfearn() by passing in the Init() lat/long.
        
};

#endif	// _UTM_PROJECTION_H
// :mode=c++:folding=explicit:
